import java.util.*;
import java.io.*;
import java.lang.Object;
import java.lang.Thread;

class Peer implements Comparable
{
    private int index;
    private String ID;
    private int capacity;
    private int load;

    // logical nodes hosted
    private int nodes [] = null;
    private int nbNodes;
    private int maxNodes = 3000;

    // Cache
    private int indiceCache;
    private String cache[] = null;
    private int tailleCache = 50;
    
    Peer (String _ID)
    {
	ID = _ID;
	nbNodes = 0;
	nodes = new int [maxNodes];
	for (int i = 0; i < maxNodes; i++)
	    nodes [i] = -1;

	cache = new String [tailleCache];
	for (int j = 0; j < tailleCache; j++)
	    cache[j] = "";

	load = 0;
	
    } // Peer

    public int compareTo(Object o)
    {
	return (getID().compareTo(((Peer)o).getID()));
    }

    public int getIndex()
    {
	return(index);
    }

    public void setIndex(int _index)
    {
	index = _index;
    }

    public void incLoad () 
    {
	load++;
    }

    public void resetLoad () 
    {
	load = 0;
    }

    public void setCapacity(int _capacity)
    {
	capacity = _capacity;
    }
    
    public int getCapacity()
    {
	return(capacity);
    }

    public String getID ()
    {
	return(ID);
    }
    
    public void setID (String _ID)
    {
	ID = _ID;
    }
    
    public int getLoad ()
    {
	return(load);
    }

    public int [] getNodes()
    {
	return(nodes);
    }

    public int getNbNodes()
    {
	return (nbNodes);
    } 

    public void clearMapping()
    {
	nbNodes = 0;
	for (int i = 0; i < maxNodes; i++)
	    nodes [i] = -1;
    } // clearMapping

    /*
     * Add a node in the list of hoted nodes 
     * respecting the circular value set order
     * @param newNode index of the node within the set of nodes
     * @param tree logical tree
     * @param ring physical ring
     */
    public void addNode (int n, Tree tree, Ring ring)
    {
	if (nbNodes > 0)
	    {
		if (tree.nodes[nodes[0]].getID().compareTo(ID) > 0)
		    // origin and node(s) before origin
		    {
			if (tree.nodes[n].getID().compareTo(ID) > 0)
			    // new node before origin
			    {
				int i = 0;
				while ((i < nbNodes) && (tree.nodes[nodes[i]].getID().compareTo(tree.nodes[n].getID()) < 0))
				    {
					if ((i != 0) && (tree.nodes[nodes[i-1]].getID().compareTo(tree.nodes[nodes[i]].getID()) > 0))
						break;
					i++;
				    }

				// shift and inser
				for (int j = nbNodes - 1; j >= i; j--) nodes[j+1] = nodes[j];
				nodes[i] = n;
			    }
			else
			    {
				int i;
				
				// searching the origin
				for (i = 1; i < nbNodes; i++)
					if (tree.nodes[nodes[i-1]].getID().compareTo(tree.nodes[nodes[i]].getID()) > 0)
					    break;

				// searching index
				for (; i < nbNodes; i++)
				    if (tree.nodes[nodes[i]].getID().compareTo(tree.nodes[n].getID()) > 0)
					break;
				
				// i == origin, shift and inser
				for (int j = nbNodes - 1; j >= i; j--) 
				    nodes[j+1] = nodes[j];
				nodes[i] = n;
			    }
		    }
		else if (tree.nodes[n].getID().compareTo(ID) > 0)
		    {
			// shift and inser
			for (int j = nbNodes - 1; j >= 0; j--) nodes[j+1] = nodes[j];
			nodes[0] = n;
		    }
		else // general case
		    {
			// find index
			int i;
			for (i = 0; i < nbNodes; i++)	
			    if (tree.nodes[nodes[i]].getID().compareTo(tree.nodes[n].getID()) > 0)
				break;
			
			// shift and inser
			for (int j = nbNodes - 1; j >= i; j--) nodes[j+1] = nodes[j];
			nodes[i] = n;
		    }
	    }
	
	else // empty set of nodes
		nodes[0] = n;

	nbNodes++;
	
    } // addNode

    public void addCache (String c)
    {
	if (indiceCache == tailleCache)
	    indiceCache = 0;
	
	cache[indiceCache++] = new String (c);
    } // addCache

    public boolean inCache(String c)
    {
	for (int i = 0; i < tailleCache; i++)
	    {
		if ((cache[i] != null) && (c.compareTo(cache[i]) == 0))
		    return (true);
	    }
	return (false);
    } // inCache    

} // Peer
