import java.util.*;
import java.io.*;

class Ring
{
    Peer [] peers;
    int nbPeers;
    static int nbPeersMax = 10000;

    Ring ()
    {
	peers = new Peer [nbPeersMax];
	for (int i = 0; i < nbPeersMax; i++)
		peers[i] = null;
	
	nbPeers = 0;
	
    } // Ring

    public void sort()
    {
	Arrays.sort(peers);

    } // sort
    
    public int getNbPeers()
    {
	return (nbPeers);
    }

    public boolean present(String idtocomp)
    {
	for (int i = 0; i < nbPeers; i++)
	    {
		if (idtocomp.compareTo(peers[i].getID()) == 0)
			return(true);
	    }
	return(false);
    } //present

    /*
     * Remove a peer at the given position
     * @param index location of the peer to be removed
     * @return void
     */
    public void removePeer(int index, Tree tree, Ring ring)
    {
	System.out.println("Removing peer " + index + " " + peers[index].getID());

	// sending the nodes to the predecessor
	
	int indexSucc;

	if (index == nbPeers - 1)
	    indexSucc = 0;
	else
	    indexSucc = index+1;
	
	int [] nodes = peers[index].getNodes();

	for (int i = 0; i < peers[index].getNbNodes(); i++)
	    {
		peers[indexSucc].addNode(nodes[i], tree, ring);
		tree.nodes[nodes[i]].setPeer(indexSucc);
	    }

	// shifting following peers and updating the peer on each node
	int i;
	for(i = index; peers[i+1] != null; i++)
	    {
		peers[i] = peers[i+1];
		peers[i].setIndex(i);
		int localNodes [] = peers[i].getNodes();
		for (int j = 0; j < peers[i].getNbNodes(); j++)
		    {
			tree.nodes[localNodes[j]].setPeer(i);
		    }
	    }
	peers[i] = null;

	// decrementing the number of peers
	nbPeers--;	
    } // removePeer

    /**
     * Add a peer at the desired position
     * update the vector
     * @param p peer to be inserted
     * @param indice specify the position of the successor of 
     * the new peer
     * @return the indice of the peer if inserted, -1 otherwise
     */
    public boolean addPeerSorted(Peer p, Tree tree)
    {
	if (present(p.getID()))
	    {
		System.out.println("ID present");
		return(false);
	    }
	else if (nbPeers == 0) // empty ring
	    {
		System.out.println("Empty ring");
		peers[nbPeers] = p;
		p.setIndex(0);
		nbPeers++;
		return(true);
	    }
	else // non empty ring
	    {
		System.out.println("Non empty ring");
		// determining first peer after origin
		int origin = 0;
		for (int i = 0; i < nbPeers-1; i++)
		    {
			if (peers[i].getID().compareTo(peers[i+1].getID()) > 0)
			    {
				origin = i+1;
				break;
			    }
		    }

		System.out.println("Origin : " + origin);

		int index;
		if (origin == 0)
		    {
			for (index = 0; ((index < nbPeers) 
					 && (peers[index].getID().compareTo(p.getID()) < 0)); 
			     index++); // index

			for (int i = nbPeers-1; i >= index; i--)
			    {
				peers[i+1] = peers[i];
				peers[i+1].setIndex(i+1);
				int [] localNodes = peers[i+1].getNodes();
				for (int j = 0; j < peers[i+1].getNbNodes(); j++)
				    {
					tree.nodes[localNodes[j]].setPeer(i+1);
				    }
			    }
			
			// inserting
			peers[index] = p; 
			p.setIndex(index); 
			nbPeers++;
		    }
		else // origin is not before the first peer
		    {
			if (peers[0].getID().compareTo(p.getID()) < 0)
			    // new peer after first peer (and before origin)
			    {
				for (index = 0; ((index < origin) && (peers[index].getID().compareTo(p.getID()) < 0)); 
				     index++); // index
				for (int i = nbPeers-1; i >= index; i--)
				    {
					peers[i+1] = peers[i];
					peers[i+1].setIndex(i+1);

					int [] localNodes = peers[i+1].getNodes();
					for (int j = 0; j < peers[i+1].getNbNodes(); j++)
					    {
						tree.nodes[localNodes[j]].setPeer(i+1);
					    }
				    }
				peers[index] = p; 
				p.setIndex(index); 
				
				nbPeers++;
			    }
			else // new peer is after the origin (before the first peer)
			    {
				for (index = origin; ((index < nbPeers) && (peers[index].getID().compareTo(p.getID()) < 0)); 
				     index++); // index
				for (int i = nbPeers-1; i >= index; i--)
				    {
					peers[i+1] = peers[i];
					peers[i+1].setIndex(i+1);
				    }
				
				peers[index] = p; p.setIndex(index); nbPeers++;
			    }
		    }
		return(true);
	    }
    } // addPeerSorted

    /**
     * Add a pair at the end of the vector
     * after verifying its the first with the given ID
     * @param p Peer to be inserted
     * @return true if inserted, false otherwise
     */
    public boolean addPeer(Peer p)
    {
	if (!present(p.getID()))
	    {
		peers[nbPeers++] = p;
		return(true);
	    }
	else
		return(false);
    } // addPeer

    public void printNodes()
    {
	int total = 0;
	for (int i = 0; i < nbPeers; i++)
	    {
		System.out.println ("\nThe peer " + i + " hosts " + 
				    peers[i].getNbNodes() + " nodes :");
		int [] noeuds = peers[i].getNodes();
		for (int j = 0; j < peers[i].getNbNodes(); j++)
		    {
			System.out.print(noeuds[j] + "\t");
		    }
		total += peers[i].getNbNodes();
	    }
	System.out.println("Total: " + total);
    }

    public void print()
    {
	System.out.println("Number of peers: " + nbPeers);

	for (int i = 0; i < getNbPeers(); i++)
	    {
		System.out.println ("Peer " + i + " " + peers[i].getID() + 
				    " capacity " + peers[i].getCapacity());
	    }
    } // afficheNoms

    public void clearMapping()
    {
	for (int i = 0; i < getNbPeers(); i++)
	    {
		peers[i].clearMapping();
	    }
    } //clearMap

    public void setCapacities()
    {
	int min = 20; // Random
	int max = 100; // Random
	Random r = new Random();

	for (int i = 0; i < getNbPeers(); i++)
	    {
		peers[i].setCapacity(r.nextInt(max-min) + min);
	    }
	
    } // setCapacities

    public void resetLoads()
    {
	for (int i = 0; i < nbPeers; i++)
	    {
		peers[i].resetLoad();
	    }
    }    
} // Ring
